"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var get = require('lodash/get');
var isEmpty = require('lodash/isEmpty');
var isFinite = require('lodash/isFinite');
var xml2js_1 = require("xml2js");
/**
 * Class implementing DataParser to fetch schema and sample data
 * using WFS requests (DescribeFeatureType resp. GetFeature)
 */
var WfsDataParser = /** @class */ (function () {
    function WfsDataParser() {
        this.service = 'WFS';
    }
    /**
     * The name Processor is passed as an option to the xml2js parser and modifies
     * the tagName. It strips all namespaces from the tags.
     *
     * @param {string} name The originial tagName
     * @return {string} The modified tagName
     */
    WfsDataParser.prototype.tagNameProcessor = function (name) {
        var prefixMatch = new RegExp(/(?!xmlns)^.*:/);
        return name.replace(prefixMatch, '');
    };
    /**
     * Generate request parameter string for a passed object
     *
     * @param {Object} params Object holding request params
     * @return {string} The URI encoded request parameter string joined with &
     */
    WfsDataParser.prototype.generateRequestParamString = function (params) {
        return Object.keys(params)
            .filter(function (key) { return !isEmpty(params[key]) || isFinite(params[key]); })
            .map(function (key) { return encodeURIComponent(key) + '=' + encodeURIComponent(params[key]); })
            .join('&');
    };
    /**
     * Map XSD datatypes (SimpleType) to json schema data stypes
     *
     * @param qualifiedXsdDataType The XSD datatype as string (including namespace)
     * @returns {JSONSchema4TypeName} The corresponding JSON schema type name
     */
    WfsDataParser.prototype.mapXsdTypeToJsonDataType = function (qualifiedXsdDataType) {
        var xsdDataType = qualifiedXsdDataType.indexOf(':') > -1
            ? qualifiedXsdDataType.split(':')[1]
            : qualifiedXsdDataType;
        switch (xsdDataType) {
            case 'string':
                return 'string';
            case 'boolean':
                return 'boolean';
            case 'float':
            case 'double':
            case 'long':
            case 'byte':
            case 'decimal':
            case 'integer':
            case 'int':
            case 'positiveInteger':
            case 'negativeInteger':
            case 'nonPositiveInteger':
            case 'nonNegativeInteger':
            case 'short':
            case 'unsignedLong':
            case 'unsignedInt':
            case 'unsignedShort':
            case 'unsignedByte':
                return 'number';
            default:
                return 'string';
        }
    };
    /**
     * Fetch schema and sample data and transforms it to the GeoStyler data model.
     *
     * Currently, WFS service must support application/json as outputFormat
     * and needs CORS headers (only needed if WFS Service is not located on the same origin
     * as the component using this parser) to be available in responses
     *
     * @param wfsConfig The parameters of the WFS
     */
    WfsDataParser.prototype.readData = function (_a) {
        var _this = this;
        var url = _a.url, version = _a.version, typeName = _a.typeName, maxFeatures = _a.maxFeatures, propertyName = _a.propertyName, featureID = _a.featureID, srsName = _a.srsName, _b = _a.fetchParams, fetchParams = _b === void 0 ? {} : _b;
        var describeFeatureTypeParams = {
            service: this.service,
            version: version,
            request: 'DescribeFeatureType',
            typenames: typeName
        };
        var getFeatureParams = {
            service: this.service,
            version: version,
            request: 'GetFeature',
            typenames: typeName,
            featureID: featureID,
            srsName: srsName,
            outputFormat: 'application/json'
        };
        if (version.startsWith('2')) {
            Object.assign(getFeatureParams, {
                count: maxFeatures
            });
        }
        else {
            Object.assign(getFeatureParams, {
                maxFeatures: maxFeatures
            });
        }
        var options = {
            tagNameProcessors: [this.tagNameProcessor]
        };
        // Fetch data schema via describe feature type
        var requestDescribeFeatureType = url + "?" + this.generateRequestParamString(describeFeatureTypeParams);
        var describeFeatureTypePromise = new Promise(function (resolve, reject) {
            fetch(requestDescribeFeatureType, fetchParams)
                .then(function (response) { return response.text(); })
                .then(function (describeFeatueTypeResult) {
                try {
                    xml2js_1.parseString(describeFeatueTypeResult, options, function (err, result) {
                        if (err) {
                            var msg = "Error while parsing DescribeFeatureType response: " + err;
                            reject(msg);
                        }
                        var attributePath = 'schema.complexType[0].complexContent[0].extension[0].sequence[0].element';
                        var attributes = get(result, attributePath);
                        var properties = {};
                        attributes.forEach(function (attr) {
                            var _a = get(attr, '$'), name = _a.name, type = _a.type;
                            if (!properties[name]) {
                                var propertyType = { type: _this.mapXsdTypeToJsonDataType(type) };
                                properties[name] = propertyType;
                            }
                        });
                        var title = get(result, 'schema.element[0].$.name');
                        var schema = {
                            type: 'object',
                            title: title,
                            properties: properties
                        };
                        resolve(schema);
                    });
                }
                catch (error) {
                    reject("Could not parse XML document: " + error);
                }
            });
        });
        // Fetch sample data via WFS GetFeature
        var requestGetFeature = url + "?" + this.generateRequestParamString(getFeatureParams);
        var getFeaturePromise = new Promise(function (resolve, reject) {
            fetch(requestGetFeature, fetchParams)
                .then(function (response) { return response.json(); })
                .then(function (getFeatureResult) {
                var fc = getFeatureResult;
                resolve(fc);
            }).catch(function (err) {
                var emptyFc = {
                    type: 'FeatureCollection',
                    features: []
                };
                reject(emptyFc);
            });
        });
        return new Promise(function (resolve, reject) {
            // Fetch features and type definition in parallel and
            // resolve if both are available
            Promise.all([describeFeatureTypePromise, getFeaturePromise])
                .then(function (results) {
                var schema = results[0], exampleFeatures = results[1];
                var data = {
                    schema: schema,
                    exampleFeatures: exampleFeatures
                };
                resolve(data);
            })
                .catch(function (error) { return reject(error); });
        });
    };
    /**
     * The name of the WfsDataParser.
     */
    WfsDataParser.title = 'WFS Data Parser';
    return WfsDataParser;
}());
exports.WfsDataParser = WfsDataParser;
exports.default = WfsDataParser;
//# sourceMappingURL=WfsDataParser.js.map